@@ -4,6 +4,7 @@ import math
 import random
 
 import numpy as np
+import datatable as dt
 
 from typing import *
 from abc import abstractmethod
@@ -35,12 +36,40 @@ data_item_type = Tuple[np.ndarray, np.ndarray]
 batch_type = Union[data_item_type, Tuple[data_item_type, np.ndarray]]
 
 
-def transpose(x: data_type) -> Union[List[List[Any]], np.ndarray]:
+def is_int(dtype: np.dtype) -> bool:
+    return np.issubdtype(dtype, np.integer)
+
+
+def is_bool(dtype: np.dtype) -> bool:
+    return np.issubdtype(dtype, np.bool)
+
+
+def is_float(dtype: np.dtype) -> bool:
+    return np.issubdtype(dtype, np.floating)
+
+
+def is_string(dtype: np.dtype) -> bool:
+    if is_int(dtype):
+        return False
+    if is_bool(dtype):
+        return False
+    if is_float(dtype):
+        return False
+    return True
+
+
+def transpose(x: data_type) -> data_type:
     if isinstance(x, np.ndarray):
         return x.T
     return list(map(list, zip(*x)))  # type: ignore
 
 
+def to_dt_data(data: data_type) -> data_type:
+    if not isinstance(data, list):
+        return data
+    return transpose(data)
+
+
 class DataTuple(NamedTuple):
     x: data_type
     y: data_type
@@ -116,6 +145,16 @@ class DataTuple(NamedTuple):
     def with_transpose(cls, x: data_type, y: data_type) -> "DataTuple":
         return DataTuple(x, y, transpose(x))
 
+    @classmethod
+    def from_dfs(cls, x_df: dt.Frame, y_df: Optional[dt.Frame]) -> "DataTuple":
+        x = x_df.to_numpy()
+        y = None if y_df is None else y_df.to_numpy()
+        if isinstance(x, np.ma.core.MaskedArray):
+            x = x.data
+        if isinstance(y, np.ma.core.MaskedArray):
+            y = y.data
+        return DataTuple.with_transpose(x, y)
+
 
 class ColumnTypes(Enum):
     NUMERICAL = "numerical"
@@ -1003,7 +1042,12 @@ __all__ = [
     "batch_type",
     "task_type_type",
     "parse_task_type",
+    "is_int",
+    "is_bool",
+    "is_float",
+    "is_string",
     "transpose",
+    "to_dt_data",
     "DataTuple",
     "ColumnTypes",
     "TaskTypes",
