@@ -1,6 +1,7 @@
 import math
 
 import numpy as np
+import datatable as dt
 
 from typing import *
 from collections import Counter
@@ -9,66 +10,40 @@ from cftool.misc import get_counter_from_arr
 
 from ..misc import *
 from ...types import *
-from ...misc.c import *
 
 
 class Recognizer(DataStructure):
+    df: dt.Frame
+    info: FeatureInfo
+    counter: Counter
+    transform_dict: transform_dict_type
+    num_unique_bound: Optional[int]
+
     def __init__(
         self,
-        column_name: str,
+        name: str,
+        is_np: bool,
         *,
         is_label: bool = False,
-        task_type: task_type_type = TaskTypes.NONE,
         is_valid: Optional[bool] = None,
-        is_string: Optional[bool] = None,
-        is_numerical: Optional[bool] = None,
-        is_categorical: Optional[bool] = None,
+        task_type: task_type_type = TaskTypes.NONE,
         config: Optional[Dict[str, Any]] = None,
     ):
-        # is_* :
-        # - `None` means no information
-        # - `False` means 'force not to *', `True` means 'force to *'
-        self.name = column_name
+        self.name = name
+        self.is_np = is_np
         self.is_label = is_label
-        self.task_type = parse_task_type(task_type)
-        if is_string is False and is_numerical is False and is_categorical is False:
-            if is_valid is None:
-                is_valid = False
-            elif is_valid:
-                raise ValueError(
-                    f"column '{column_name}' is neither string, numerical nor "
-                    "categorical, but it is still set to be valid"
-                )
         self.is_valid = is_valid
-        self.is_string = is_string
-        self.is_numerical = is_numerical
-        self.is_categorical = is_categorical
-        self._num_unique_bound: Optional[int]
+        self.task_type = parse_task_type(task_type)
         self._init_config(config)
-        self._info: FeatureInfo
-        self._counter: Counter
-        self._transform_dict: transform_dict_type
 
     def __str__(self) -> str:
         return f"Recognizer({self.info.column_type})"
 
     __repr__ = __str__
 
-    @property
-    def info(self) -> FeatureInfo:
-        return self._info
-
-    @property
-    def counter(self) -> Counter:
-        return self._counter
-
-    @property
-    def transform_dict(self) -> transform_dict_type:
-        return self._transform_dict
-
     @property
     def num_unique_values(self) -> Union[int, float]:
-        if self._info.is_numerical:
+        if self.info.is_numerical:
             return math.inf
         return len(self._transformed_unique_values)
 
@@ -78,17 +53,17 @@ class Recognizer(DataStructure):
         self.config = config
         self._num_thresh = config.setdefault("numerical_threshold", 0.5)
         default_bound = 128
-        self._num_unique_bound = config.setdefault("num_unique_bound", default_bound)
+        self.num_unique_bound = config.setdefault("num_unique_bound", default_bound)
         self._truncate_ratio = config.setdefault("truncate_ratio", 0.99)
-        if self._num_unique_bound is None:
+        if self.num_unique_bound is None:
             default_fuse_threshold = 1.0 / default_bound
         else:
-            default_fuse_threshold = 1.0 / self._num_unique_bound
+            default_fuse_threshold = 1.0 / self.num_unique_bound
         self._fuse_thresh = config.setdefault("fuse_threshold", default_fuse_threshold)
-        if self._num_unique_bound is None:
+        if self.num_unique_bound is None:
             default_fuse_fix = int(default_bound // 2)
         else:
-            default_fuse_fix = int(round(0.5 * self._num_unique_bound))
+            default_fuse_fix = int(round(0.5 * self.num_unique_bound))
         self._num_fuse_fix = config.setdefault("num_fuse_fix", default_fuse_fix)
 
     def _make_invalid_info(
@@ -97,7 +72,7 @@ class Recognizer(DataStructure):
         contains_nan: bool,
         nan_mask: np.ndarray,
     ) -> "Recognizer":
-        self._info = FeatureInfo(
+        self.info = FeatureInfo(
             contains_nan,
             None,
             nan_mask=nan_mask,
@@ -121,7 +96,7 @@ class Recognizer(DataStructure):
             is_valid=is_valid,
             need_transform=True,
             column_type=ColumnTypes.STRING,
-            num_unique_bound=self._num_unique_bound,
+            num_unique_bound=self.num_unique_bound,
             unique_values_sorted_by_counts=unique_values,
             sorted_counts=sorted_counts,
         )
@@ -134,7 +109,7 @@ class Recognizer(DataStructure):
         return FeatureInfo(
             None,
             None,
-            num_unique_bound=self._num_unique_bound,
+            num_unique_bound=self.num_unique_bound,
             unique_values_sorted_by_counts=unique_values,
             sorted_counts=sorted_counts,
         )
@@ -192,24 +167,20 @@ class Recognizer(DataStructure):
 
     def _check_string_column(
         self,
-        flat_arr: flat_arr_type,
+        df: dt.Frame,
+        np_flat: np.ndarray,
+        nan_mask: np.ndarray,
     ) -> Tuple[bool, Optional[FeatureInfo]]:
-        if self.is_numerical or self.is_categorical or self.is_string is False:
+        if not is_string(df.stype.dtype):
             return False, None
         msg: Optional[str] = None
-        is_reg_label = self.is_label and self.task_type.is_reg
-        if self.is_string:
-            if is_reg_label:
-                raise ValueError("task_type is REGRESSION but labels are set to string")
-        else:
-            all_numeric = is_all_numeric(flat_arr)
-            if is_reg_label and not all_numeric:
-                msg = "task_type is REGRESSION but labels are not all numeric"
-                raise ValueError(msg)
-            if all_numeric:
-                return False, None
-        self._counter = get_counter_from_arr(flat_arr)
-        most_common = self._counter.most_common()
+        np_flat[nan_mask] = "nan"
+        self.counter = get_counter_from_arr(np_flat)
+        most_common = sorted(
+            self.counter.most_common(),
+            key=lambda pair: pair[::-1],
+            reverse=True,
+        )
         unique_values = [elem[0] for elem in most_common]
         sorted_counts = np.array([elem[1] for elem in most_common])
         num_unique_values = len(unique_values)
@@ -219,21 +190,24 @@ class Recognizer(DataStructure):
                 "are the SAME. It'll be excluded since it might be redundant"
             )
             return True, self._make_string_info(None, False, msg)
-        if not self.is_valid and num_unique_values == len(flat_arr):
+        if not self.is_valid and num_unique_values == len(np_flat):
             msg = (
                 f"all values in column {self.name}, which tends to be string column, "
                 "are DIFFERENT. It'll be excluded since it might be redundant"
             )
             return True, self._make_string_info(None, False, msg)
-        if num_unique_values >= 1e3:
+        unique_ratio = num_unique_values / len(np_flat)
+        if not self.is_valid and unique_ratio >= 0.5:
             msg = (
-                f"TOO MANY unique values occurred in column {self.name} "
-                f"({num_unique_values:^12d})"
+                f"unique values occurred in column {self.name}, which tends to be "
+                f"string column, are TOO MANY (ratio={unique_ratio:8.6f}). "
+                "It'll be excluded since it might be redundant"
             )
+            return True, self._make_string_info(None, False, msg)
         pack = self._get_transform_dict(False, unique_values, sorted_counts)
-        self._transform_dict, self._transformed_unique_values = pack
+        self.transform_dict, self._transformed_unique_values = pack
         return True, self._make_string_info(
-            flat_arr,
+            np_flat,
             True,
             msg,
             np.array(unique_values),
@@ -259,10 +233,7 @@ class Recognizer(DataStructure):
                 "since it might be redundant"
             )
             return "exclude", msg
-        if (
-            not self.is_categorical
-            and num_unique_values >= self._num_thresh * num_samples
-        ):
+        if num_unique_values >= self._num_thresh * num_samples:
             msg = (
                 f"TOO MANY unique values occurred in column {self.name} "
                 f"({num_unique_values:^12d}) which tends to be categorical column, "
@@ -273,40 +244,54 @@ class Recognizer(DataStructure):
         return "keep", None
 
     def _generate_categorical_transform_dict(self) -> None:
-        unique_values = self._info.unique_values_sorted_by_counts
-        sorted_counts = self._info.sorted_counts
+        unique_values = self.info.unique_values_sorted_by_counts
+        sorted_counts = self.info.sorted_counts
         assert unique_values is not None and sorted_counts is not None
         values = list(map(float, unique_values.tolist()))
-        pack = self._get_transform_dict(True, values, sorted_counts, self._info)
+        pack = self._get_transform_dict(True, values, sorted_counts, self.info)
         transform_dict, self._transformed_unique_values = pack
-        if self._info.contains_nan:
+        if self.info.contains_nan:
             num_transformed_unique = len(self._transformed_unique_values)
             transform_dict["nan"] = num_transformed_unique
             self._transformed_unique_values.append(num_transformed_unique)
-        self._transform_dict = transform_dict
+        self.transform_dict = transform_dict
 
-    def fit(self, flat_arr: flat_arr_type) -> "Recognizer":
+    def fit(self, df: dt.Frame) -> "Recognizer":
         msg: Optional[str]
+        dtype = df.stype.dtype
+        # check name
+        if len(df.names) != 1 or df.names[0] != self.name:
+            raise ValueError(
+                f"df.name ({df.names[0]}) is not identical with "
+                f"Recognizer.name ({self.name})"
+            )
+        # check valid
         if self.is_valid is False:
-            self._info = FeatureInfo(
+            self.info = FeatureInfo(
                 None,
                 None,
                 False,
                 msg=f"current column ({self.name}) is forced to be invalid",
             )
             return self
-        is_string, info = self._check_string_column(flat_arr)
-        if is_string:
-            assert info is not None
-            self._info = info
-            return self
-        if isinstance(flat_arr[0], (str, np.str_)):
-            np_flat = flat_arr_to_float32(flat_arr)
-            if np_float_type != np.float32:
-                np_flat = np.asarray(np_flat, np_float_type)
+        # check string
+        df_data = df.to_numpy()
+        if isinstance(df_data, np.ma.core.MaskedArray):
+            np_flat = df_data.data.ravel()
+            nan_mask = df_data.mask.ravel()
         else:
-            np_flat = np.asarray(flat_arr, np_float_type)
-        nan_mask = np.isnan(np_flat)
+            np_flat = df_data.ravel()
+            nan_mask = np.zeros_like(np_flat, dtype=np.bool)
+        is_string_, info = self._check_string_column(df, np_flat, nan_mask)
+        if is_string_:
+            assert info is not None
+            self.info = info
+            return self
+        # check bool
+        if is_bool(dtype):
+            dtype = np.int64
+            np_flat = np_flat.astype(dtype)
+        # gather info
         valid_mask = ~nan_mask
         np_flat_valid = np_flat[valid_mask]
         np_flat_valid_int = np_flat_valid.astype(np_int_type)
@@ -317,9 +302,11 @@ class Recognizer(DataStructure):
             nan_mask = None
         # check whether all nan or not
         if num_valid_samples == 0:
+            if self.is_label:
+                raise ValueError("label column is detected as all-invalid")
             if self.is_valid:
                 np_flat = np.zeros_like(np_flat)
-                self._info = FeatureInfo(
+                self.info = FeatureInfo(
                     contains_nan,
                     np_flat,
                     nan_mask=np.zeros_like(nan_mask),
@@ -332,59 +319,47 @@ class Recognizer(DataStructure):
             )
             return self._make_invalid_info(msg, contains_nan, nan_mask)
         # check whether it's a numerical column
-        all_int = np.allclose(np_flat_valid, np_flat_valid_int)
-        is_classification_label = self.is_label and self.task_type.is_clf
-        if (
-            self.is_numerical
-            or self.is_numerical is None
-            and (
-                self.is_categorical is False
-                or self.is_label
-                and self.task_type.is_reg
-                or self.is_categorical is None
-                and not all_int
-            )
-        ):
-            if not is_classification_label:
-                msg, is_valid = None, True
-                if (
-                    self.is_valid is None
-                    and np_flat_valid.max() - np_flat_valid.min() <= 1e-8
-                ):
-                    is_valid = False
-                    msg = (
-                        f"all values in column {self.name}, which tends to be "
-                        "numerical column, are ALL CLOSE. It'll be excluded since "
-                        "it might be redundant"
-                    )
-                self._info = FeatureInfo(
-                    contains_nan,
-                    np_flat,
-                    nan_mask=nan_mask,
-                    is_valid=is_valid,
-                    msg=msg,
+        not_int = is_float(dtype) and not np.allclose(np_flat_valid, np_flat_valid_int)
+        is_reg_label = self.is_label and self.task_type.is_reg
+        if not_int or is_reg_label:
+            msg, is_valid = None, True
+            if (
+                self.is_valid is None
+                and np_flat_valid.max() - np_flat_valid.min() <= 1e-8
+            ):
+                is_valid = False
+                msg = (
+                    f"all values in column {self.name}, which tends to be "
+                    "numerical column, are ALL CLOSE. It'll be excluded since "
+                    "it might be redundant"
                 )
-                return self
+            self.info = FeatureInfo(
+                contains_nan,
+                np_flat.astype(np_float_type),
+                nan_mask=nan_mask,
+                is_valid=is_valid,
+                msg=msg,
+            )
+            return self
         # deal with categorical column
-        np_flat_categorical = np_flat_valid_int if all_int else np_flat_valid
-        min_feat = np_flat_categorical.min()
-        np_flat_categorical -= min_feat
-        max_feature_value = np_flat_categorical.max()
-        if not all_int or max_feature_value > 1e6:
-            unique_values, counts = np.unique(np_flat_categorical, return_counts=True)
+        min_feat = np_flat_valid_int.min()
+        np_flat_valid_int -= min_feat
+        max_feature_value = np_flat_valid_int.max()
+        if max_feature_value > 1e6:
+            unique_values, counts = np.unique(np_flat_valid_int, return_counts=True)
             num_unique_values = len(unique_values)
             need_transform = unique_values[-1] != num_unique_values - 1
         else:
-            counts = np.bincount(np_flat_categorical)
+            counts = np.bincount(np_flat_valid_int)
             unique_values = np.nonzero(counts)[0]
             num_unique_values = len(unique_values)
             need_transform = num_unique_values != len(counts)
             counts = counts[unique_values]
         need_transform = min_feat != 0 or need_transform
         status, msg = self._check_exclude_categorical(num_samples, num_unique_values)
-        if not is_classification_label and status != "keep":
+        if not self.is_label and status != "keep":
             if status == "numerical":
-                self._info = FeatureInfo(
+                self.info = FeatureInfo(
                     contains_nan,
                     np_flat,
                     nan_mask=nan_mask,
@@ -400,15 +375,19 @@ class Recognizer(DataStructure):
             unique_values = np.append(unique_values, [float("nan")])
             sorted_counts = np.append(sorted_counts, [float(num_nan)])
             counter_dict["nan"] = num_nan
+            # we need to go back to float32 to inject nan so we can
+            #  correctly transform categorical columns into 'ordered' indices later on
+            np_flat = np_flat.copy().astype(np.float32)
+            np_flat[nan_mask] = np.nan
         need_transform = need_transform or contains_nan
-        self._counter = Counter(counter_dict)
-        self._info = FeatureInfo(
+        self.counter = Counter(counter_dict)
+        self.info = FeatureInfo(
             contains_nan,
             np_flat,
             nan_mask=nan_mask,
             need_transform=need_transform,
             column_type=ColumnTypes.CATEGORICAL,
-            num_unique_bound=self._num_unique_bound,
+            num_unique_bound=self.num_unique_bound,
             unique_values_sorted_by_counts=unique_values,
             sorted_counts=sorted_counts,
         )
@@ -433,7 +412,7 @@ class Recognizer(DataStructure):
 
     @classmethod
     def loads(cls, instance_dict: Dict[str, Any], **kwargs: Any) -> "Recognizer":
-        recognizer = cls("")
+        recognizer = cls("", False)
         recognizer.__dict__.update(instance_dict)
         return recognizer
 
